@@ -26,15 +26,6 @@ sub filter (@) { #{{{
 	return $params{content};
 } # }}}
 
-sub scrub ($) { #{{{
-	if (IkiWiki::Plugin::htmlscrubber->can("sanitize")) {
-		return IkiWiki::Plugin::htmlscrubber::sanitize(content => shift);
-	}
-	else {
-		return shift;
-	}
-} #}}}
-
 sub preprocess (@) { #{{{
 	if (! @_) {
 		return "";
@@ -54,9 +45,9 @@ sub preprocess (@) { #{{{
 
 	if ($key eq 'link') {
 		if (%params) {
-			$meta{$page}.=scrub("<link href=\"".encode_entities($value)."\" ".
+			$meta{$page}.="<link href=\"".encode_entities($value)."\" ".
 				join(" ", map { encode_entities($_)."=\"".encode_entities(decode_entities($params{$_}))."\"" } keys %params).
-				" />\n");
+				" />\n";
 		}
 		else {
 			# hidden WikiLink
@@ -64,37 +55,15 @@ sub preprocess (@) { #{{{
 		}
 	}
 	elsif ($key eq 'title') {
-		$title{$page}=encode_entities($value);
+		$title{$page}=$value;
 	}
 	elsif ($key eq 'permalink') {
 		$permalink{$page}=$value;
-		$meta{$page}.=scrub("<link rel=\"bookmark\" href=\"".encode_entities($value)."\" />\n");
-	}
-	elsif ($key eq 'stylesheet') {
-		my $rel=exists $params{rel} ? $params{rel} : "alternate stylesheet";
-		my $title=exists $params{title} ? $params{title} : $value;
-		# adding .css to the value prevents using any old web
-		# editable page as a stylesheet
-		my $stylesheet=bestlink($page, $value.".css");
-		if (! length $stylesheet) {
-			return "[[meta ".gettext("stylesheet not found")."]]";
-		}
-		$meta{$page}.='<link href="'.$stylesheet.
-			'" rel="'.encode_entities($rel).
-			'" title="'.encode_entities($title).
-			"\" style=\"text/css\" />\n";
-	}
-	elsif ($key eq 'openid') {
-		if (exists $params{server}) {
-			$meta{$page}.='<link href="'.encode_entities($params{server}).
-				"\" rel=\"openid.server\" />\n";
-		}
-		$meta{$page}.='<link href="'.encode_entities($value).
-			"\" rel=\"openid.delegate\" />\n";
+		$meta{$page}.="<link rel=\"bookmark\" href=\"".encode_entities($value)."\" />\n";
 	}
 	else {
-		$meta{$page}.=scrub("<meta name=\"".encode_entities($key).
-			"\" content=\"".encode_entities($value)."\" />\n");
+		$meta{$page}.="<meta name=\"".encode_entities($key).
+			"\" content=\"".encode_entities($value)."\" />\n";
 		if ($key eq 'author') {
 			$author{$page}=$value;
 		}
